function [x,fval,y]=convexbounds(W,K,p,q,w0,k0,minmax,tol,solver);
% Computes arbitrage bounds on the basket (w0^Tx-K0)^+ given
% prices of basket  (w_i^Tx-K_)^+
% Forward prices are supposed to be given by q

disp(' ');
disp('--- Start Cvx Bound ---');
n=length(q);
m=size(W,1);
euc=eye(n);
% Add simple forwards to the price constraints
Wbuf=euc;p=[p;q];K=[K;zeros(n,1)];
% Add basket forwards to the price constraints
for i=1:m
    if min(max(abs(Wbuf-ones(size(Wbuf,1),1)*W(i,:))'))>0
        p=[p;W(i,:)*q];Wbuf=[Wbuf;W(i,:)];K=[K;0];
    end
end
p=[p;w0*q];Wbuf=[Wbuf;w0];K=[K;0];
W=[W;Wbuf];
%num2str([W,K,p],'%4.3f ')

% Set problem params
m=size(W,1);
A=[];b=[];
Aeq=[];beqlow=[];bequp=[];beq=[];

% Monotonicity constraints (sign of the subgradients)
for i=1:(m+1)
    A=[A;sparse([zeros(n,(i-1)*(n+1)),-eye(n),zeros(n,1+(m+1-i)*(n+1)+1)])];
    b=[b;zeros(n,1)];
    A=[A;sparse([zeros(1,(i-1)*(n+1)+n),1,zeros(1,(m+1-i)*(n+1)),0])];
    b=[b;0];
    A=[A;sparse([zeros(1,(i-1)*(n+1)+n),-1,zeros(1,(m+1-i)*(n+1)),0])];
    b=[b;1];
end

% Price must be positive
A=[A;sparse([zeros(1,(m+1)*(n+1)),-1])];
b=[b;0];

if strcmp(solver,'sedumi')
    % Pricing constraints (Bid Ask)
    for j=1:m
        Aeq=[Aeq;sparse([zeros(1,(j-1)*(n+1)),W(j,:),K(j),zeros(1,(m+1-j)*(n+1)),0])];
        bequp=[bequp;p(j)*(1+tol)];
        beqlow=[beqlow;p(j)*(1-tol)];
    end
    Aeq=[Aeq;sparse([zeros(1,m*(n+1)),w0,k0,-1])];
    bequp=[bequp;0];
    beqlow=[beqlow;0];

    A=[A;Aeq;-Aeq];
    b=[b;bequp;-beqlow];
    clear Aeq;
else
    for j=1:m
        Aeq=[Aeq;sparse([zeros(1,(j-1)*(n+1)),W(j,:),K(j),zeros(1,(m+1-j)*(n+1)),0])];
        beq=[beq;p(j)];
    end
    Aeq=[Aeq;sparse([zeros(1,m*(n+1)),w0,k0,-1])];
    beq=[beq;0];
end

% Convexity constraints
%for i=1:(m+1)
%   for j=1:m
%        A=[A;[zeros(1,(i-1)*(n+1)),W(j,:),K(j),zeros(1,(m+1-i)*(n+1)),0]];
%        b=[b;p(j)];
%    end
%    A=[A;[zeros(1,(i-1)*(n+1)),w0,k0,zeros(1,(m+1-i)*(n+1)),-1]];
%    b=[b;0];
%end

b=sparse(b);
% Convexity constraints
[As,bs]=CvxMat(W,K,p,w0,k0);
A=[A;As];
b=[b;bs];

% Objective
c=sparse([zeros((n+1)*(m+1),1);1]);

if strcmp(solver,'sedumi')
    % Solve using sedumi
    c=minmax*c;
    disp(['Problem size: ',num2str(size(A))]);
    K.l=size(A,1);
    pars.fid=1;
    [x,y]=sedumi(A',c,b,K,pars);
    fval=minmax*c'*y;
else
    % Solve using LINPROG
    c=-minmax*c;
    disp(['Problem size: ',num2str(size(A))]);
    %options = optimset('Display','iter','TolFun',1e-6);
    options = optimset('TolFun',1e-10);
    %options = optimset('LargeScale', 'off', 'Simplex', 'on');
    [y,fval,EXITFLAG,OUTPUT,x]=LINPROG(c,A,b,Aeq,beq,[],[],[],options);
    fval=fval*-minmax;
end
    

